﻿Imports System.IO
Imports System.ComponentModel

Namespace System.Data

   Public Class DatasetXmlAdapter : Inherits DatasetAdapterBase

      Public DataFile As FileInfo

      Public Sub New(_Dts As DataSet)
         DataSet = _Dts
#If DEBUG Then
         DataFile = New FileInfo(New DirectoryInfo("..\..\").Combine(_Dts.GetType.Name & ".xml"))
#Else
         DataFile = New FileInfo(New DirectoryInfo(Application.LocalUserAppDataPath).Combine(_Dts.GetType.Name & ".xml"))
#End If
      End Sub

      Protected Overrides Sub _Save(attaches As List(Of DataTableAttach))
         _DataSet.AcceptChanges()
         Using _DataSet.SuspendConstraints
            ReadWriteSelective(FileAccess.Write)
         End Using
      End Sub

		Protected Overrides Sub _Fill(tbls As Collections.Generic.IEnumerable(Of DataTable))
			If tbls.FirstOrDefault.NotNull Then Throw New NotSupportedException(
					"Bei DatasetOnly werden immer alle Tabellen befüllt - bestimmte Tabellen anzugeben ist sinnlos")
			DataFile.Refresh()
			If DataFile.Exists() Then
				Using _DatasetAttach.BeginSuspendAll
					ReadWriteSelective(FileAccess.Read)
					DataSet.AcceptChanges()
				End Using
			Else
				Msg("DatenFile{n}{}{n}nicht gefunden.{n}Evtl. können Sie dennoch fortfahren, und es wird angelegt, wenn Sie abspeichern".FormatX(DataFile.FullName))
			End If
		End Sub

		Private Sub ReadWriteSelective(access As FileAccess)
			'skipInfos sagt über jede Tabelle aus, ob Persistierung auslassen
			Dim skipInfos = _DataSet.Tables.Cast(Of DataTable).ToList(Function(tb) tb.Attach.Persistance = Persistance.No)
			If skipInfos.Any(Function(b) b) Then '                                              mindestens eine Tabelle auslassen
				Using clone = New DataSet, rd = New StringReader(_DataSet.GetXmlSchema)
					clone.ReadXmlSchema(rd)
					If access = FileAccess.Read Then '                                   clone einlesen und Tabellen selektiv nach _Dataset kopieren
						ReadWrite(clone, DataFile, FileAccess.Read)
						CopyTablesSelective(clone, _DataSet, skipInfos)
					Else '                                                     _Dataset-Tabellen selektiv nach clone kopieren, dann clone abspeichern
						CopyTablesSelective(_DataSet, clone, skipInfos)
						ReadWrite(clone, DataFile, FileAccess.Write)
					End If
				End Using
			Else
				If access = FileAccess.Read Then _DataSet.Clear()
				ReadWrite(_DataSet, DataFile, access) ' ohne Persistenz-Auslassung ist einfacher und schneller
			End If
		End Sub

		Private Shared Sub CopyTablesSelective(src As DataSet, dest As DataSet, skipInfos As List(Of Boolean))
			'skipInfos sagt über jede Tabelle aus, ob Kopieren auslassen
			'ausgelassene Tabellen können im ZielDataset ihre Daten unverändert behalten - so kann man zB hardcodet DefaultData bereitstellen
			For i = 0 To src.Tables.Count - 1
				If skipInfos(i) Then Continue For
				With dest.Tables(i)
					.Clear()
					.Merge(src.Tables(i))
				End With
			Next
		End Sub

		Private Shared Sub ReadWrite(dts As DataSet, fi As FileInfo, access As FileAccess)
         If fi.Extension.ToLower = ".txt" Then
            If access = FileAccess.Read Then
               dts.LoadCompact(fi.FullName)
            Else
               dts.SaveCompact(fi.FullName)
            End If
         Else
            If access = FileAccess.Read Then
               dts.ReadXml(fi.FullName)
            Else
               dts.WriteXml(fi.FullName)
            End If
         End If
      End Sub

   End Class

End Namespace